package com.muyu.system.controller;

import com.muyu.common.core.constant.UserConstants;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.system.domain.SysMenu;
import com.muyu.system.domain.model.SysMenuAddModel;
import com.muyu.system.domain.model.SysMenuListModel;
import com.muyu.system.domain.model.SysMenuUpdModel;
import com.muyu.system.domain.rep.SysMenuAddReq;
import com.muyu.system.domain.rep.SysMenuListReq;
import com.muyu.system.domain.rep.SysMenuUpdReq;
import com.muyu.system.domain.resp.RoleMenuTreeResp;
import com.muyu.system.domain.resp.SysMenuListResp;
import com.muyu.system.domain.vo.RouterVo;
import com.muyu.system.domain.vo.TreeSelect;
import com.muyu.system.service.SysMenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 菜单信息
 *
 * @author muyu
 */
@RestController
@RequestMapping("/menu")
public class SysMenuController extends BaseController {
    @Autowired
    private SysMenuService menuService;

    /**
     * 获取菜单列表
     */
    @RequiresPermissions("system:menu:list")
    @PostMapping("/list")
    public Result<List<SysMenuListResp>> list (@RequestBody SysMenuListReq sysMenuListReq) {
        Long userId = SecurityUtils.getUserId();
        List<SysMenuListModel> menus = menuService.selectMenuList(SysMenuListModel.of(sysMenuListReq), userId);
        return success(menus.stream()
                .map(SysMenuListResp::buildSysMenuListResp)
                .toList());
    }

    /**
     * 根据菜单编号获取详细信息
     */
    @RequiresPermissions("system:menu:query")
    @GetMapping(value = "/{menuId}")
    public Result<SysMenu> getInfo (@PathVariable("menuId") Long menuId) {
        return success(menuService.selectMenuById(menuId));
    }

    /**
     * 获取菜单下拉树列表
     */
    @GetMapping("/treeselect")
    public Result<List<TreeSelect>> treeselect (SysMenuListReq sysMenuListReq) {
        Long userId = SecurityUtils.getUserId();
        List<SysMenuListModel> sysMenus = menuService.selectMenuList(SysMenuListModel.of(sysMenuListReq), userId);
        return success(menuService.buildMenuTreeSelect(sysMenus));
    }

    /**
     * 加载对应角色菜单列表树
     */
    @GetMapping(value = "/roleMenuTreeselect/{roleId}")
    public Result<RoleMenuTreeResp> roleMenuTreeselect (@PathVariable("roleId") Long roleId) {
        Long userId = SecurityUtils.getUserId();
        List<SysMenuListModel> menus = menuService.selectMenuList(userId);
        return Result.success(
                RoleMenuTreeResp.builder()
                        .menus(menuService.buildMenuTreeSelect(menus))
                        .checkedKeys(menuService.selectMenuListByRoleId(roleId))
                        .build()
        );
    }

    /**
     * 新增菜单
     */
    @RequiresPermissions("system:menu:add")
    @Log(title = "菜单管理", businessType = BusinessType.INSERT)
    @PostMapping
    public Result<String> add (@Validated @RequestBody SysMenuAddReq sysMenuAddReq) {
        SysMenuAddModel sysMenuAddModel = SysMenuAddModel.of(sysMenuAddReq);
        if (!menuService.checkMenuNameUnique(sysMenuAddModel)) {
            return error("新增菜单'" + sysMenuAddModel.getMenuName() + "'失败，菜单名称已存在");
        } else if (UserConstants.YES_FRAME.equals(sysMenuAddModel.getIsFrame()) && !StringUtils.ishttp(sysMenuAddModel.getPath())) {
            return error("新增菜单'" + sysMenuAddModel.getMenuName() + "'失败，地址必须以http(s)://开头");
        }
        SysMenu menu = SysMenu.buildSysMenu(sysMenuAddReq);
        menu.setCreateBy(SecurityUtils.getUsername());
        menuService.insertMenu(SysMenuAddModel.of(sysMenuAddReq));
        return Result.success();
    }

    /**
     * 修改菜单
     */
    @RequiresPermissions("system:menu:edit")
    @Log(title = "菜单管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result<String> edit (@Validated @RequestBody SysMenuUpdReq sysMenuUpdReq) {
        SysMenuAddModel sysMenuAddModel = SysMenuAddModel.buildSysMenuAddModel(sysMenuUpdReq);
        if (!menuService.checkMenuNameUnique(sysMenuAddModel)) {
            return error("新增菜单'" + sysMenuAddModel.getMenuName() + "'失败，菜单名称已存在");
        } else if (UserConstants.YES_FRAME.equals(sysMenuAddModel.getIsFrame()) && !StringUtils.ishttp(sysMenuAddModel.getPath())) {
            return error("新增菜单'" + sysMenuAddModel.getMenuName() + "'失败，地址必须以http(s)://开头");
        }
        SysMenu menu = SysMenu.buildSysMenu(sysMenuAddModel);
        menu.setCreateBy(SecurityUtils.getUsername());
        menu.setUpdateBy(SecurityUtils.getUsername());
        menuService.updateMenu(SysMenuUpdModel.of(sysMenuUpdReq));
        return Result.success();
    }

    /**
     * 删除菜单
     */
    @RequiresPermissions("system:menu:remove")
    @Log(title = "菜单管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{menuId}")
    public Result<String> remove (@PathVariable("menuId") Long menuId) {
        if (menuService.hasChildByMenuId(menuId)) {
            return warn("存在子菜单,不允许删除");
        }
        if (menuService.checkMenuExistRole(menuId)) {
            return warn("菜单已分配,不允许删除");
        }
        menuService.deleteMenuById(menuId);
        return Result.success();
    }

    /**
     * 获取路由信息
     *
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public Result<List<RouterVo>> getRouters () {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return success(menuService.buildMenus(menus));
    }
}
